( cd usr/lib ; rm -rf libxalanMsg.so.110 )
( cd usr/lib ; ln -sf libxalanMsg.so.110.0 libxalanMsg.so.110 )
( cd usr/lib ; rm -rf libxalanMsg.so )
( cd usr/lib ; ln -sf libxalanMsg.so.110.0 libxalanMsg.so )
( cd usr/lib ; rm -rf libxalan-c.so )
( cd usr/lib ; ln -sf libxalan-c.so.110.0 libxalan-c.so )
( cd usr/lib ; rm -rf libxalan-c.so.110 )
( cd usr/lib ; ln -sf libxalan-c.so.110.0 libxalan-c.so.110 )
